require 'cmessages'

RSpec.describe Cmessages do
    it 'has a version number' do
        expect(Cmessages::VERSION).not_to be nil
    end

    context 'Class Accessor Method' do
        before(:each) do
            @cm = Cmessages.new
            @cmnc = Cmessages.new(:use_colours => false)
            @cmnp = Cmessages.new(:use_prefixes => false)
            @cmncnp = Cmessages.new(:use_colours => false, :use_prefixes => false)
            @cmcustome = Cmessages.new(:error_prefix => '{ Error }')
            @cmcustomw = Cmessages.new(:warning_prefix => '{ Warning }')
            @cmcustoms = Cmessages.new(:success_prefix => '{ Success }')
            @cmcustomi = Cmessages.new(:info_prefix => '{ Info }')
        end

        context 'Error prefix validation' do
            context 'default (colour and prefix)' do
                specify { expect { @cm.error('foo') }.to output("\e[0;91;49m[ Error ] foo\e[0m\n").to_stdout }
            end
            context 'no colour' do
                specify { expect { @cmnc.error('foo') }.to output("[ Error ] foo\n").to_stdout }
            end
            context 'no prefix' do
                specify { expect { @cmnp.error('foo') }.to output("\e[0;91;49mfoo\e[0m\n").to_stdout }
            end
            context 'no prefix and no colour' do
                specify { expect { @cmncnp.error('foo') }.to output("foo\n").to_stdout }
            end
        end

        context 'Warning Prefixes' do
            context 'default (colour and prefix)' do
                specify { expect { @cm.warning('foo') }.to output("\e[0;93;49m[ Warning ] foo\e[0m\n").to_stdout }
            end
            context 'no colour' do
                specify { expect { @cmnc.warning('foo') }.to output("[ Warning ] foo\n").to_stdout }
            end
            context 'no prefix' do
                specify { expect { @cmnp.warning('foo') }.to output("\e[0;93;49mfoo\e[0m\n").to_stdout }
            end
            context 'no prefix and no colour' do
                specify { expect { @cmncnp.warning('foo') }.to output("foo\n").to_stdout }
            end
        end

        context 'Success Prefixes' do
            context 'default (colour and prefix)' do
                specify { expect { @cm.success('foo') }.to output("\e[0;92;49m[ Success ] foo\e[0m\n").to_stdout }
            end
            context 'no colour' do
                specify { expect { @cmnc.success('foo') }.to output("[ Success ] foo\n").to_stdout }
            end
            context 'no prefix' do
                specify { expect { @cmnp.success('foo') }.to output("\e[0;92;49mfoo\e[0m\n").to_stdout }
            end
            context 'no prefix and no colour' do
                specify { expect { @cmncnp.success('foo') }.to output("foo\n").to_stdout }
            end
        end

        context 'Info Prefixes' do
            context 'default (colour and prefix)' do
                specify { expect { @cm.info('foo') }.to output("\e[0;96;49m[ Info ] foo\e[0m\n").to_stdout }
            end
            context 'no colour' do
                specify { expect { @cmnc.info('foo') }.to output("[ Info ] foo\n").to_stdout }
            end
            context 'no prefix' do
                specify { expect { @cmnp.info('foo') }.to output("\e[0;96;49mfoo\e[0m\n").to_stdout }
            end
            context 'no prefix and no colour' do
                specify { expect { @cmncnp.info('foo') }.to output("foo\n").to_stdout }
            end
        end

        context 'Custom Prefixes' do
            context 'error prefix' do
                specify { expect { Cmessages.new(:error_prefix => '{ Error }').error('foo') }.to output("\e[0;91;49m{ Error } foo\e[0m\n").to_stdout }
            end
            context 'warning prefix' do
                specify { expect { Cmessages.new(:warning_prefix => '{ Warning }').warning('foo') }.to output("\e[0;93;49m{ Warning } foo\e[0m\n").to_stdout }
            end
            context 'success prefix' do
                specify { expect { Cmessages.new(:success_prefix => '{ Success }').success('foo') }.to output("\e[0;92;49m{ Success } foo\e[0m\n").to_stdout }
            end
            context 'info prefix' do
                specify { expect { Cmessages.new(:info_prefix => '{ Info }').info('foo') }.to output("\e[0;96;49m{ Info } foo\e[0m\n").to_stdout }
            end
        end

        context 'Custom Colours' do
            context 'error colour' do
                specify { expect { Cmessages.new(:error_colour => :blue).error('foo') }.to output("\e[0;34;49m[ Error ] foo\e[0m\n").to_stdout }
            end
            context 'warning colour' do
                specify { expect { Cmessages.new(:warning_colour => :white).warning('foo') }.to output("\e[0;37;49m[ Warning ] foo\e[0m\n").to_stdout }
            end
            context 'success colour' do
                specify { expect { Cmessages.new(:success_colour => :black).success('foo') }.to output("\e[0;30;49m[ Success ] foo\e[0m\n").to_stdout }
            end
            context 'info colour' do
                specify { expect { Cmessages.new(:info_colour => :magenta).info('foo') }.to output("\e[0;35;49m[ Info ] foo\e[0m\n").to_stdout }
            end
        end

        context 'Multi-line Strings' do
            context 'error' do
                specify { expect { @cmnc.error("This has multiple\nlines in it") }.to output("[ Error ] This has multiple\nlines in it\n").to_stdout }
            end
            context 'warning' do
                specify { expect { @cmnc.warning("This has multiple\nlines in it") }.to output("[ Warning ] This has multiple\nlines in it\n").to_stdout }
            end
            context 'success' do
                specify { expect { @cmnc.success("This has multiple\nlines in it") }.to output("[ Success ] This has multiple\nlines in it\n").to_stdout }
            end
            context 'info' do
                specify { expect { @cmnc.info("This has multiple\nlines in it") }.to output("[ Info ] This has multiple\nlines in it\n").to_stdout }
            end
        end

        context 'Multi-line Strings (including blank lines)' do
            context 'error' do
                specify { expect { @cmnc.error("This has multiple\n\nlines in it") }.to output("[ Error ] This has multiple\n\nlines in it\n").to_stdout }
            end
            context 'warning' do
                specify { expect { @cmnc.warning("This has multiple\n\nlines in it") }.to output("[ Warning ] This has multiple\n\nlines in it\n").to_stdout }
            end
            context 'success' do
                specify { expect { @cmnc.success("This has multiple\n\nlines in it") }.to output("[ Success ] This has multiple\n\nlines in it\n").to_stdout }
            end
            context 'info' do
                specify { expect { @cmnc.info("This has multiple\n\nlines in it") }.to output("[ Info ] This has multiple\n\nlines in it\n").to_stdout }
            end
        end

        context 'Empty Strings' do
            context 'error' do
                specify { expect { @cm.error('') }.to output('').to_stdout }
            end
            context 'warning' do
                specify { expect { @cm.warning('') }.to output('').to_stdout }
            end
            context 'success' do
                specify { expect { @cm.success('') }.to output('').to_stdout }
            end
            context 'info' do
                specify { expect { @cm.info('') }.to output('').to_stdout }
            end
        end
    end

    context 'String Accessor Method' do
        context 'Error prefix validation' do
            context 'default (colour and prefix)' do
                specify { expect { 'foo'.error }.to output("\e[0;91;49m[ Error ] foo\e[0m\n").to_stdout }
            end
            context 'no colour' do
                specify { expect { 'foo'.error(:use_colours => false) }.to output("[ Error ] foo\n").to_stdout }
            end
            context 'no prefix' do
                specify { expect { 'foo'.error(:use_prefixes => false) }.to output("\e[0;91;49mfoo\e[0m\n").to_stdout }
            end
            context 'no prefix and no colour' do
                specify { expect { 'foo'.error(:use_colours => false, :use_prefixes => false) }.to output("foo\n").to_stdout }
            end
        end

        context 'Warning Prefixes' do
            context 'default (colour and prefix)' do
                specify { expect { 'foo'.warning }.to output("\e[0;93;49m[ Warning ] foo\e[0m\n").to_stdout }
            end
            context 'no colour' do
                specify { expect { 'foo'.warning(:use_colours => false) }.to output("[ Warning ] foo\n").to_stdout }
            end
            context 'no prefix' do
                specify { expect { 'foo'.warning(:use_prefixes => false) }.to output("\e[0;93;49mfoo\e[0m\n").to_stdout }
            end
            context 'no prefix and no colour' do
                specify { expect { 'foo'.warning(:use_colours => false, :use_prefixes => false) }.to output("foo\n").to_stdout }
            end
        end

        context 'Success Prefixes' do
            context 'default (colour and prefix)' do
                specify { expect { 'foo'.success }.to output("\e[0;92;49m[ Success ] foo\e[0m\n").to_stdout }
            end
            context 'no colour' do
                specify { expect { 'foo'.success(:use_colours => false) }.to output("[ Success ] foo\n").to_stdout }
            end
            context 'no prefix' do
                specify { expect { 'foo'.success(:use_prefixes => false) }.to output("\e[0;92;49mfoo\e[0m\n").to_stdout }
            end
            context 'no prefix and no colour' do
                specify { expect { 'foo'.success(:use_colours => false, :use_prefixes => false) }.to output("foo\n").to_stdout }
            end
        end

        context 'Info Prefixes' do
            context 'default (colour and prefix)' do
                specify { expect { 'foo'.info }.to output("\e[0;96;49m[ Info ] foo\e[0m\n").to_stdout }
            end
            context 'no colour' do
                specify { expect { 'foo'.info(:use_colours => false) }.to output("[ Info ] foo\n").to_stdout }
            end
            context 'no prefix' do
                specify { expect { 'foo'.info(:use_prefixes => false) }.to output("\e[0;96;49mfoo\e[0m\n").to_stdout }
            end
            context 'no prefix and no colour' do
                specify { expect { 'foo'.info(:use_colours => false, :use_prefixes => false) }.to output("foo\n").to_stdout }
            end
        end

        context 'Custom Prefixes' do
            context 'error prefix' do
                specify { expect { 'foo'.error(:error_prefix => '{ Error }') }.to output("\e[0;91;49m{ Error } foo\e[0m\n").to_stdout }
            end
            context 'warning prefix' do
                specify { expect { 'foo'.warning(:warning_prefix => '{ Warning }') }.to output("\e[0;93;49m{ Warning } foo\e[0m\n").to_stdout }
            end
            context 'success prefix' do
                specify { expect { 'foo'.success(:success_prefix => '{ Success }') }.to output("\e[0;92;49m{ Success } foo\e[0m\n").to_stdout }
            end
            context 'info prefix' do
                specify { expect { 'foo'.info(:info_prefix => '{ Info }') }.to output("\e[0;96;49m{ Info } foo\e[0m\n").to_stdout }
            end
        end

        context 'Custom Colours' do
            context 'error colour' do
                specify { expect { 'foo'.error(:error_colour => :blue) }.to output("\e[0;34;49m[ Error ] foo\e[0m\n").to_stdout }
            end
            context 'warning colour' do
                specify { expect { 'foo'.warning(:warning_colour => :white) }.to output("\e[0;37;49m[ Warning ] foo\e[0m\n").to_stdout }
            end
            context 'success colour' do
                specify { expect { 'foo'.success(:success_colour => :black) }.to output("\e[0;30;49m[ Success ] foo\e[0m\n").to_stdout }
            end
            context 'info colour' do
                specify { expect { 'foo'.info(:info_colour => :magenta) }.to output("\e[0;35;49m[ Info ] foo\e[0m\n").to_stdout }
            end
        end

        context 'Multi-line Strings' do
            context 'error' do
                specify { expect { "This has multiple\nlines in it".error(:use_colours => false) }.to output("[ Error ] This has multiple\nlines in it\n").to_stdout }
            end
            context 'warning' do
                specify { expect { "This has multiple\nlines in it".warning(:use_colours => false) }.to output("[ Warning ] This has multiple\nlines in it\n").to_stdout }
            end
            context 'success' do
                specify { expect { "This has multiple\nlines in it".success(:use_colours => false) }.to output("[ Success ] This has multiple\nlines in it\n").to_stdout }
            end
            context 'info' do
                specify { expect { "This has multiple\nlines in it".info(:use_colours => false) }.to output("[ Info ] This has multiple\nlines in it\n").to_stdout }
            end
        end

        context 'Multi-line Strings (including blank lines)' do
            context 'error' do
                specify { expect { "This has multiple\n\nlines in it".error(:use_colours => false) }.to output("[ Error ] This has multiple\n\nlines in it\n").to_stdout }
            end
            context 'warning' do
                specify { expect { "This has multiple\n\nlines in it".warning(:use_colours => false) }.to output("[ Warning ] This has multiple\n\nlines in it\n").to_stdout }
            end
            context 'success' do
                specify { expect { "This has multiple\n\nlines in it".success(:use_colours => false) }.to output("[ Success ] This has multiple\n\nlines in it\n").to_stdout }
            end
            context 'info' do
                specify { expect { "This has multiple\n\nlines in it".info(:use_colours => false) }.to output("[ Info ] This has multiple\n\nlines in it\n").to_stdout }
            end
        end

        context 'Empty Strings' do
            context 'error' do
                specify { expect { ''.error }.to output('').to_stdout }
            end
            context 'warning' do
                specify { expect { ''.warning }.to output('').to_stdout }
            end
            context 'success' do
                specify { expect { ''.success }.to output('').to_stdout }
            end
            context 'info' do
                specify { expect { ''.info }.to output('').to_stdout }
            end
        end
    end
end
